"""
Contains a set of utility functions to validate
input arguments
"""


def not_none(obj, msg):
    """
    Raises ValueError in case the given input is None
    @param x: Any, the input value
    @param msg: str, message in the raised Exception (if any)
    @return: None
    """
    if obj is None:
        raise ValueError(msg)
